/*
 * AlibException.h
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#ifndef ALIB_EXCEPTION_H_
#define ALIB_EXCEPTION_H_

#include "CommonException.h"
#include <deque>
#include "../sax/Token.h"

#include "../object/ObjectBase.h"

namespace exception {

/**
 * Basic exception from which are derived all other exceptions.
 * Contains reason why the exception occured.
 */
class AlibException : public exception::CommonException, public alib::ObjectBase {
public:
	explicit AlibException ( );

	explicit AlibException ( std::string cause );

	explicit AlibException ( std::string cause, std::string backtrace );

	virtual alib::ObjectBase * clone ( ) const;

	virtual alib::ObjectBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const AlibException & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static AlibException parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace exception */

namespace std {

template < >
struct compare < ::exception::AlibException > {
	int operator ()( const::exception::AlibException & first, const::exception::AlibException & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* ALIB_EXCEPTION_H_ */
