/*
 * CommonException.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#include "CommonException.h"

#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <sstream>

#include <execinfo.h>
#ifdef DEBUG
	#include "../debug/simpleStacktrace.h"
#endif

namespace exception {

CommonException::CommonException ( ) {
	#ifdef DEBUG
		this->backtrace = std::simpleStacktrace();
	#else
		this->backtrace = "";
	#endif

	this->whatMessage += this->backtrace;
}

CommonException::CommonException ( std::string cause ) : CommonException { } {
	this->whatMessage += this->cause;

	this->cause = std::move(cause);
}

CommonException::CommonException ( std::string cause, std::string backtrace ) {
	this->whatMessage += this->backtrace;
	this->whatMessage += this->cause;

	this->backtrace = std::move(backtrace);
	this->cause = std::move(cause);
}

CommonException::~CommonException ( ) noexcept {

}

const char * CommonException::what ( ) const noexcept {
	return whatMessage.c_str ( );
}

const std::string & CommonException::getCause ( ) const {
	return cause;
}

const std::string & CommonException::getBacktrace ( ) const {
	return backtrace;
}

void CommonException::operator>>(std::ostream& os) const {
	os << this->whatMessage;
}

CommonException::operator std::string() const {
	return this->whatMessage;
}

} /* namespace exception */

