/*
 * CommonException.h
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#ifndef COMMON_EXCEPTION_H_
#define COMMON_EXCEPTION_H_

#include <exception>
#include <string>

namespace exception {

/**
 * Basic exception from which are derived all other exceptions.
 * Contains reason why the exception occured.
 */
class CommonException : public std::exception {
protected:
	std::string cause;
	std::string backtrace;
	std::string whatMessage;

public:
	explicit CommonException ( );

	explicit CommonException ( std::string cause );

	explicit CommonException ( std::string cause, std::string backtrace );

	virtual ~CommonException ( ) noexcept;

	/**
	 * @return reason why the exception occured
	 */
	const char * what ( ) const noexcept;

	/**
	 * @return reason why the exception occured
	 */
	const std::string & getCause ( ) const;

	/**
	 * @return reason why the exception occured
	 */
	const std::string & getBacktrace ( ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

};

} /* namespace exception */

#endif /* COMMON_EXCEPTION_H_ */
