/*
 * Author: Radovan Cerveny
 */

#ifndef MEMORY_DATA_FRAME_HPP_
#define MEMORY_DATA_FRAME_HPP_

#include <deque>
#include "../../sax/Token.h"
#include "../MeasurementTypes.hpp"

namespace measurements {

struct MeasurementFrame;
struct MemoryDataFrame;

struct MemoryHint {
	using frame_type = MemoryDataFrame;
	enum class Type {
		NEW, DELETE
	};

	Type   type;
	size_t size;
};

struct MemoryDataFrame {
	size_t start_heap_usage;
	size_t end_heap_usage;
	size_t high_watermark;

	size_t current_heap_usage;

	static void init ( unsigned, measurements::stealth_vector < MeasurementFrame > & );
	static void update ( unsigned, measurements::stealth_vector < MeasurementFrame > & );
	static void hint ( unsigned, measurements::stealth_vector < MeasurementFrame > &, MemoryHint );
	static void xml_tokens ( std::deque < sax::Token > &, unsigned, const measurements::stealth_vector < MeasurementFrame > & );
};

std::ostream & operator <<( std::ostream &, const MemoryDataFrame & );
}

#endif /* MEMORY_DATA_FRAME_HPP_ */
