/*
 * Author: Radovan Cerveny
 */

#include "fdstream.hpp"

namespace std {

const int FAIL_FD = -1;

fdstreambuf::fdstreambuf ( int fd ) : fd ( fd ) {
	setp ( buff.begin ( ), buff.end ( ) - 1 );
}

fdstreambuf::~fdstreambuf ( ) {
	sync ( );
}

bool fdstreambuf::flush ( ) {
	std::ptrdiff_t n = pptr ( ) - pbase ( );

	ssize_t osz = write ( fd, buff.data ( ), n );

	pbump ( -n );

	return osz != -1;
}

fdstreambuf::int_type fdstreambuf::overflow ( fdstreambuf::int_type ch ) {
	if ( ch == traits_type::eof ( ) ) return traits_type::eof ( );

	buff.back ( ) = ch;

	pbump ( 1 );

	if ( !flush ( ) ) return traits_type::eof ( );

	return ch;
}

int fdstreambuf::sync ( ) {
	return flush ( ) ? 0 : -1;
}

fdaccessor::fdaccessor ( int new_fd, int fallback_fd ) {
	if ( fcntl ( new_fd, F_GETFD ) == 0 ) {
		fd = new_fd;
		redirected = true;
	} else if ( fcntl ( fallback_fd, F_GETFD ) == 0 ) {
		fd = fallback_fd;
		redirected = false;
	} else {
		fd = -1;
		redirected = false;
	}
}

int fdaccessor::get_fd ( ) const {
	return fd;
}

bool fdaccessor::is_redirected ( ) const {
	return redirected;
}

ofdstream::ofdstream ( int fd, int fallback_fd = FAIL_FD) : fda ( fd, fallback_fd ), fdbuf ( fda.get_fd ( ) ) {
	this->init ( & fdbuf );

	if ( fda.get_fd ( ) == -1 )
		setstate ( ios_base::failbit );
}

ofdstream::~ofdstream ( ) {
}

bool ofdstream::is_redirected ( ) const {
	return fda.is_redirected ( );
}

}
